/*
	Speccy Tracks Remake!
	
	Tricks by Spectra
	- Allow the ability to "trick" off of the floor.
	- Also includes JumpDrifting.
*/

local cv_jumpdrift = CV_RegisterVar({
    name = "jumpdrift",
    defaultvalue = "Off",
    flags = CV_NETVAR,
    PossibleValue = CV_OnOff
})

local cv_tricks = CV_RegisterVar({
    name = "spec_tricks",
    defaultvalue = "On",
    flags = CV_NETVAR,
    PossibleValue = CV_OnOff
})

local cv_braketricks = CV_RegisterVar({
    name = "tricks_button",
    defaultvalue = "Brake",
    flags = CV_NETVAR|CV_SAVE,
    PossibleValue = {Drift = 1, Brake = 0}
})

local function CanTrick(p)
    if not P_IsObjectOnGround(p.mo) -- Must be off the floor
	and not p.kartstuff[k_respawn] -- Can't be respawning
	and not p.kartstuff[k_spinouttimer] -- Must not be in pain
	and not p.kartstuff[k_pogospring] -- Must not be using the Pogo Spring item
	and not p.mo.driftjumped -- Must not have jumped with jumpdrift
	and not p.mo.tricked -- Must not have tricked already
	and (p.mo.skin ~= "srb2sonic") --That'd be dumb
	and (p.playerstate == PST_LIVE) -- Must be alive
	and (p.speed > 17*FRACUNIT) -- Can't be going slow
		return p.mo.tricktimer > 0
	end

	return false
end 

local function DoTrick(p)
	S_StartSound(p.mo, sfx_s1c9)
	p.mo.tricked = true

	p.mo.trickang = 1

    p.mo.trickaura = P_SpawnMobj(p.mo.x, p.mo.y, p.mo.z, MT_INVULNFLASH)
    p.mo.trickaura.target = p.mo 
    p.mo.trickaura.color = p.mo.color
    p.mo.trickaura.sprite = SPR_KINF
	p.mo.trickaura.destscale = 3*FRACUNIT
	p.mo.trickaura.fuse = TICRATE/3

	K_PlayBoostTaunt(p.mo)

	local sparkle
	for i = 1, 10 do
		sparkle = P_SpawnMobj(p.mo.x, p.mo.y, p.mo.z, MT_KARMAFIREWORK)
		sparkle.color = p.mo.color
		sparkle.colorized = true

		K_MatchGenericExtraFlags(sparkle, p.mo)

		sparkle.momx = p.mo.momx + P_RandomRange(-5, 5)*FRACUNIT
		sparkle.momy = p.mo.momy + P_RandomRange(-5, 5)*FRACUNIT
		sparkle.momz = p.mo.momz + P_RandomRange(-5, 5)*FRACUNIT

		sparkle.fuse = 35
	end
end

K_AddPThink(function(p)
	if P_IsObjectOnGround(p.mo) then p.mo.tricktimer = 25 end

	-- Force tricking
	if p.mo.timeuntilforcetrick then
		p.mo.timeuntilforcetrick = $-1
		if p.mo.timeuntilforcetrick <= 0 then
			p.mo.timeuntilforcetrick = nil
			if p.mo.tricked then p.mo.forcetrick = true end
		end
	end

	-- Trick time limit
	if p.mo.tricktimer and p.mo.tricktimer > 0 then p.mo.tricktimer = $-1 end

	-- Jump Drift
	if (p.buttons.drift == 1)
		if cv_jumpdrift.value and not p.mo.driftjumped and P_IsObjectOnGround(p.mo)
			S_StartSound(p.mo, sfx_s3k56)
			if (p.mo.eflags & MFE_UNDERWATER)
				P_SetObjectMomZ(p.mo, FRACUNIT)
			else
				P_SetObjectMomZ(p.mo, 3*FRACUNIT)
			end

			p.mo.driftjumped = true
		end
	end

	-- Tricks
	if (cv_braketricks.value == 0 and p.buttons.brake == 1) or (cv_braketricks.value == 1 and p.buttons.drift == 1) then
		if cv_tricks.value and CanTrick(p) then
			if p.mo.gliding then
				if p.mo.upglide >= 5 then DoTrick(p) end
			else
				DoTrick(p)
			end
		end
	end

	if p.mo.tricked and cv_tricks.value
		if P_IsObjectOnGround(p.mo) or (p.mo.gliding and p.mo.upglide <= 0) or (p.mo.forcetrick)
			S_StartSound(p.mo, sfx_s23c)
			p.kartstuff[k_driftboost] = $1 + 30
			p.frameangle = p.mo.angle

			p.mo.tricked = nil
			p.mo.trickang = nil
			p.mo.forcetrick = nil

			if p.mo.gliding then p.mo.glidespeed = $+7*FRACUNIT end
		end

		if (p.mo.eflags & MFE_JUSTBOUNCEDWALL) or P_PlayerInPain(p)
			p.mo.tricked = nil
			p.mo.trickang = nil
			p.mo.forcetrick = nil
			p.mo.timeuntilforcetrick = nil
		end

		if p.playerstate ~= PST_LIVE then p.mo.tricked = nil end

		if p.mo.trickang then
			p.frameangle = p.mo.angle + p.mo.trickang
			p.mo.trickang = $+ANG20
		end
	end

	if p.mo.trickaura and p.mo.trickaura.valid then
		P_TeleportMove(p.mo.trickaura, p.mo.x, p.mo.y, p.mo.z)
	end

	if cv_jumpdrift.value and P_IsObjectOnGround(p.mo) and not (p.cmd.buttons & BT_DRIFT)
		p.frameangle = p.mo.angle
		p.mo.driftjumped = nil
	end
end)